@@ -447,7 +447,8 @@ public class SweetHome3D extends HomeApplication {
             // If application has no more home
             if (getHomes().isEmpty() 
                 && (!OperatingSystem.isMacOSX()
-                    || !Boolean.getBoolean("apple.laf.useScreenMenuBar"))) {
+                    || !Boolean.getBoolean("apple.laf.useScreenMenuBar")
+                    || OperatingSystem.isJavaVersionGreaterOrEqual("1.9"))) {
               // If SingleInstanceService is available, remove the listener that was added on it
               if (singleInstanceService != null) {
                 singleInstanceService.removeSingleInstanceListener(singleInstanceListener);
@@ -518,7 +519,10 @@ public class SweetHome3D extends HomeApplication {
       System.setProperty("apple.awt.application.name", applicationName);
       if (System.getProperty("apple.laf.useScreenMenuBar") == null) {
         // Use Mac OS X screen menu bar for frames menu bar
-        System.setProperty("apple.laf.useScreenMenuBar", "true");
+        // except for bundles under macOS 10.13
+        System.setProperty("apple.laf.useScreenMenuBar", 
+            String.valueOf(OperatingSystem.compareVersions(System.getProperty("os.version"), "10.13") < 0
+                || MacOSXConfiguration.isScreenMenuBarSupported()));
       }
       // Force the use of Quartz under Mac OS X for better Java 2D rendering performance
       System.setProperty("apple.awt.graphics.UseQuartz", "true");
