@@ -294,8 +294,8 @@ public class SweetHome3DApplet extends JApplet {
       applicationClassName = getApplicationClassName();
       Class sweetHome3DAppletClass = SweetHome3DApplet.class;
       List java3DFiles = new ArrayList();
-      if (!System.getProperty("os.name").startsWith("Mac OS X")
-          || System.getProperty("java.version").startsWith("1.5")) {
+      if (System.getProperty("os.name").startsWith("Mac OS X")
+          && System.getProperty("java.version").startsWith("1.5")) {
         java3DFiles.addAll(Arrays.asList(new String [] {
             "j3dcore.jar", // Main Java 3D jars
             "vecmath.jar",
@@ -308,15 +308,15 @@ public class SweetHome3DApplet extends JApplet {
             "macosx/libjogl_cg.jnilib"}));
       } else {
         java3DFiles.addAll(Arrays.asList(new String [] {
-            "macosx/java3d-1.6/j3dcore.jar", // Mac OS X Java 3D 1.6 jars and DLLs
-            "macosx/java3d-1.6/vecmath.jar",
-            "macosx/java3d-1.6/j3dutils.jar",
-            "macosx/java3d-1.6/gluegen.jar", 
-            "macosx/java3d-1.6/jogl-java3d.jar",
-            "macosx/java3d-1.6/libgluegen-rt.jnilib",
-            "macosx/java3d-1.6/libjogl_desktop.jnilib",
-            "macosx/java3d-1.6/libnativewindow_awt.jnilib",
-            "macosx/java3d-1.6/libnativewindow_macosx.jnilib"}));
+            "java3d-1.6/j3dcore.jar", // Mac OS X Java 3D 1.6 jars and DLLs
+            "java3d-1.6/vecmath.jar",
+            "java3d-1.6/j3dutils.jar",
+            "java3d-1.6/gluegen-rt.jar", 
+            "java3d-1.6/jogl-java3d.jar",
+            "java3d-1.6/macosx/libgluegen-rt.jnilib",
+            "java3d-1.6/macosx/libjogl_desktop.jnilib",
+            "java3d-1.6/macosx/libnativewindow_awt.jnilib",
+            "java3d-1.6/macosx/libnativewindow_macosx.jnilib"}));
         try {
           // Disable JOGL library loader
           System.setProperty("jogamp.gluegen.UseTempJarCache", "false");
@@ -326,16 +326,25 @@ public class SweetHome3DApplet extends JApplet {
         }
       }
       if ("64".equals(System.getProperty("sun.arch.data.model"))) {
-        java3DFiles.add("linux/x64/libj3dcore-ogl.so"); // Linux 64 bits DLLs
-        java3DFiles.add("windows/x64/j3dcore-ogl.dll"); // Windows 64 bits DLLs
+        java3DFiles.addAll(Arrays.asList(new String [] {
+          "java3d-1.6/linux/amd64/libgluegen-rt.so", // Linux 64 bits DLLs for Java 3D 1.6
+          "java3d-1.6/linux/amd64/libjogl_desktop.so",
+          "java3d-1.6/linux/amd64/libnativewindow_awt.so",
+          "java3d-1.6/linux/amd64/libnativewindow_x11.so",
+          "java3d-1.6/windows/amd64/gluegen-rt.dll", // Windows 64 bits DLLs for Java 3D 1.6
+          "java3d-1.6/windows/amd64/jogl_desktop.dll",
+          "java3d-1.6/windows/amd64/nativewindow_awt.dll",
+          "java3d-1.6/windows/amd64/nativewindow_win32.dll"}));
       } else {
         java3DFiles.addAll(Arrays.asList(new String [] {
-            "linux/i386/libj3dcore-ogl.so", // Linux 32 bits DLLs
-            "linux/i386/libj3dcore-ogl-cg.so", // Windows 32 bits DLLs
-            "windows/i386/j3dcore-d3d.dll",
-            "windows/i386/j3dcore-ogl.dll",
-            "windows/i386/j3dcore-ogl-cg.dll",
-            "windows/i386/j3dcore-ogl-chk.dll"}));
+          "java3d-1.6/linux/i586/libgluegen-rt.so", // Linux 32 bits DLLs for Java 3D 1.6
+          "java3d-1.6/linux/i586/libjogl_desktop.so",
+          "java3d-1.6/linux/i586/libnativewindow_awt.so",
+          "java3d-1.6/linux/i586/libnativewindow_x11.so",
+          "java3d-1.6/windows/i586/gluegen-rt.dll", // Windows 32 bits DLLs for Java 3D 1.6
+          "java3d-1.6/windows/i586/jogl_desktop.dll",
+          "java3d-1.6/windows/i586/nativewindow_awt.dll",
+          "java3d-1.6/windows/i586/nativewindow_win32.dll"}));
       }
       
       List applicationPackages = new ArrayList(Arrays.asList(new String [] {
