@@ -43,6 +43,23 @@ import com.eteks.sweethome3d.tools.ExtensionsClassLoader;
  *     it will be read relative to applet document base. By default, this parameter is 
  *     equal to <code>default.sh3d</code>.</li>
  *     
+ *     <li><code>level</code> specifies the name of the level that should be displayed. 
+ *     Note that selecting a level may have a different effect according to the 
+ *     <i>Display all levels / Selected level</i> options in the displayed home 
+ *     and whether visitor eyes should be adjusted to selected level. 
+ *     By default, the chosen level (if there are some) is one used when home was saved.</li>
+ *     
+ *     <li><code>camera</code> specifies the name of the stored point of view that should be selected. 
+ *     By default, the camera is one used when home was saved.</li>
+ *     
+ *     <li><code>selectableLevels</code> specifies a comma separated list of level names 
+ *     that will be added to the contextual menu of the 3D view, to let the user select a level.</li>
+ *     
+ *     <li><code>selectableCameras</code> specifies a comma separated list of stored viewpoint names 
+ *     that will be added to the contextual menu of the 3D view, to let the user select a point of view.
+ *     If both <code>selectableLevels</code> and <code>selectableCameras</code> are specified, a
+ *     separator will be added in the contextual menu to separate both lists.</li>
+ *     
  *     <li><code>ignoreCache</code> specifies whether home file may be read from Java 
  *     cache or not. 
  *     <br>If its value is <code>true</code>, then each time the applet is launched the
@@ -51,10 +68,12 @@ import com.eteks.sweethome3d.tools.ExtensionsClassLoader;
  *     
  *     <li><code>navigationPanel</code> specifies whether navigation arrows should be 
  *     displayed or not. By default, its value is <code>false</code>. 
- *     Use <code>true</code> to display navigation panel.</li></ul>
+ *     Use <code>true</code> to display navigation panel.</li>
+ *
+ *     <li><code>activateCameraSwitchKey</code> specifies whether the space bar should be used
+ *     or not to switch between aerial view and virtual visitor view. 
+ *     By default, its value is <code>true</code>.</li></ul>
  *     
- * <p>Use space bar to switch between aerial view and virtual visitor view.</p>
- * 
  * <p>The bytecode of this class is Java 1.1 compatible to be able to notify users that 
  * it requires Java 5 when it's run under an old JVM.
  *     
@@ -127,8 +146,8 @@ public class SweetHome3DViewer extends JApplet {
     try {
       Class sweetHome3DViewerClass = SweetHome3DViewer.class;
       List java3DFiles = new ArrayList();
-      if (!System.getProperty("os.name").startsWith("Mac OS X")
-          || System.getProperty("java.version").startsWith("1.5")) {
+      if (System.getProperty("os.name").startsWith("Mac OS X")
+          && System.getProperty("java.version").startsWith("1.5")) {
         java3DFiles.addAll(Arrays.asList(new String [] {
             "j3dcore.jar", // Main Java 3D jars
             "vecmath.jar",
@@ -141,30 +160,38 @@ public class SweetHome3DViewer extends JApplet {
             "macosx/libjogl_cg.jnilib"}));
       } else {
         java3DFiles.addAll(Arrays.asList(new String [] {
-            "macosx/java3d-1.6/j3dcore.jar", // Mac OS X Java 3D 1.6 jars and DLLs
-            "macosx/java3d-1.6/vecmath.jar",
-            "macosx/java3d-1.6/j3dutils.jar",
-            "macosx/java3d-1.6/gluegen.jar", 
-            "macosx/java3d-1.6/jogl-java3d.jar",
-            "macosx/java3d-1.6/libgluegen-rt.jnilib",
-            "macosx/java3d-1.6/libjogl_desktop.jnilib",
-            "macosx/java3d-1.6/libnativewindow_awt.jnilib",
-            "macosx/java3d-1.6/libnativewindow_macosx.jnilib"}));
+            "java3d-1.6/j3dcore.jar", // Mac OS X Java 3D 1.6 jars and DLLs
+            "java3d-1.6/vecmath.jar",
+            "java3d-1.6/j3dutils.jar",
+            "java3d-1.6/gluegen-rt.jar", 
+            "java3d-1.6/jogl-java3d.jar",
+            "java3d-1.6/macosx/libgluegen-rt.jnilib",
+            "java3d-1.6/macosx/libjogl_desktop.jnilib",
+            "java3d-1.6/macosx/libnativewindow_awt.jnilib",
+            "java3d-1.6/macosx/libnativewindow_macosx.jnilib"}));
         // Disable JOGL library loader
         System.setProperty("jogamp.gluegen.UseTempJarCache", "false");
-        System.setProperty("com.eteks.sweethome3d.j3d.useOffScreen3DView", "true");
       }
       if ("64".equals(System.getProperty("sun.arch.data.model"))) {
-        java3DFiles.add("linux/x64/libj3dcore-ogl.so"); // Linux 64 bits DLLs
-        java3DFiles.add("windows/x64/j3dcore-ogl.dll"); // Windows 64 bits DLLs
+        java3DFiles.addAll(Arrays.asList(new String [] {
+          "java3d-1.6/linux/amd64/libgluegen-rt.so", // Linux 64 bits DLLs for Java 3D 1.6
+          "java3d-1.6/linux/amd64/libjogl_desktop.so",
+          "java3d-1.6/linux/amd64/libnativewindow_awt.so",
+          "java3d-1.6/linux/amd64/libnativewindow_x11.so",
+          "java3d-1.6/windows/amd64/gluegen-rt.dll", // Windows 64 bits DLLs for Java 3D 1.6
+          "java3d-1.6/windows/amd64/jogl_desktop.dll",
+          "java3d-1.6/windows/amd64/nativewindow_awt.dll",
+          "java3d-1.6/windows/amd64/nativewindow_win32.dll"}));
       } else {
         java3DFiles.addAll(Arrays.asList(new String [] {
-            "linux/i386/libj3dcore-ogl.so", // Linux 32 bits DLLs
-            "linux/i386/libj3dcore-ogl-cg.so", // Windows 32 bits DLLs
-            "windows/i386/j3dcore-d3d.dll",
-            "windows/i386/j3dcore-ogl.dll",
-            "windows/i386/j3dcore-ogl-cg.dll",
-            "windows/i386/j3dcore-ogl-chk.dll"}));
+          "java3d-1.6/linux/i586/libgluegen-rt.so", // Linux 32 bits DLLs for Java 3D 1.6
+          "java3d-1.6/linux/i586/libjogl_desktop.so",
+          "java3d-1.6/linux/i586/libnativewindow_awt.so",
+          "java3d-1.6/linux/i586/libnativewindow_x11.so",
+          "java3d-1.6/windows/i586/gluegen-rt.dll", // Windows 32 bits DLLs for Java 3D 1.6
+          "java3d-1.6/windows/i586/jogl_desktop.dll",
+          "java3d-1.6/windows/i586/nativewindow_awt.dll",
+          "java3d-1.6/windows/i586/nativewindow_win32.dll"}));
       }
 
       List applicationPackages = new ArrayList(Arrays.asList(new String [] {
