@@ -32,6 +32,10 @@ import java.io.InterruptedIOException;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.Writer;
+import java.net.JarURLConnection;
+import java.net.URISyntaxException;
+import java.net.URL;
+import java.net.URLConnection;
 import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
 import java.text.NumberFormat;
@@ -39,6 +43,7 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Locale;
@@ -47,6 +52,8 @@ import java.util.zip.ZipEntry;
 import java.util.zip.ZipOutputStream;
 
 import javax.imageio.ImageIO;
+import javax.imageio.ImageReader;
+import javax.imageio.stream.ImageInputStream;
 import javax.media.j3d.Appearance;
 import javax.media.j3d.ColoringAttributes;
 import javax.media.j3d.Geometry;
@@ -73,6 +80,7 @@ import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.TexCoordGeneration;
 import javax.media.j3d.Texture;
+import javax.media.j3d.TextureAttributes;
 import javax.media.j3d.Transform3D;
 import javax.media.j3d.TransformGroup;
 import javax.media.j3d.TransparencyAttributes;
@@ -109,6 +117,7 @@ public class OBJWriter extends FilterWriter {
   private Map<ComparableAppearance, String> appearances = 
       new LinkedHashMap<ComparableAppearance, String>();
   private Map<Texture, File> textures = new HashMap<Texture, File>();
+  private List<URL>          copiedTextures = new ArrayList<URL>();
   
   /**
    * Create an OBJ writer for the given file, with no header and default precision.
@@ -366,9 +375,14 @@ public class OBJWriter extends FilterWriter {
         this.out.write("g " + objectName + "\n");
         
         TexCoordGeneration texCoordGeneration = null;
+        Transform3D textureTransform = new Transform3D();
         if (this.mtlFileName != null) {
           if (appearance != null) {
             texCoordGeneration = appearance.getTexCoordGeneration();
+            TextureAttributes textureAttributes = appearance.getTextureAttributes();
+            if (textureAttributes != null) {
+              textureAttributes.getTextureTransform(textureTransform);
+            }
             ComparableAppearance comparableAppearance = new ComparableAppearance(appearance);
             String appearanceName = this.appearances.get(comparableAppearance);
             if (appearanceName == null) {
@@ -376,7 +390,7 @@ public class OBJWriter extends FilterWriter {
               try {
                 appearanceName = appearance.getName();
               } catch (NoSuchMethodError ex) {
-                // Don't reuse appearance name with Java 3D < 1.4 where getName was added              }
+                // Don't reuse appearance name with Java 3D < 1.4 where getName was added
               }
               if (appearanceName == null || !accept(appearanceName)) {
                 appearanceName = objectName;
@@ -398,9 +412,30 @@ public class OBJWriter extends FilterWriter {
               if (texture != null) {
                 File textureFile = this.textures.get(texture);
                 if (textureFile == null) {
+                  String fileExtension = "png";
+                  URL textureUrl = (URL)texture.getUserData();
+                  if (textureUrl instanceof URL) {
+                    InputStream in = null;
+                    try {
+                      // Find the format of the texture image
+                      in = openStream(textureUrl);
+                      ImageInputStream imageIn = ImageIO.createImageInputStream(in);
+                      Iterator<ImageReader> imageReaders = ImageIO.getImageReaders(imageIn);
+                      if (imageReaders.hasNext()) {
+                        ImageReader reader = (ImageReader)imageReaders.next();
+                        fileExtension = reader.getFormatName().toLowerCase();
+                        // Store the URL to copy its image content directly when appearances are saved 
+                        this.copiedTextures.add(textureUrl);
+                      }
+                    } catch (IOException ex) {
+                      if (in != null) {
+                        in.close();
+                      }
+                    }
+                  }
                   // Store texture
                   textureFile = new File(this.mtlFileName.substring(0, this.mtlFileName.length() - 4) 
-                      + "_" + appearanceName + ".png");
+                      + "_" + appearanceName + "." + fileExtension);
                   this.textures.put(texture, textureFile);
                 }
               }
@@ -422,12 +457,38 @@ public class OBJWriter extends FilterWriter {
         // Write object geometries
         for (int i = 0, n = shape.numGeometries(); i < n; i++) {
           writeNodeGeometry(shape.getGeometry(i), parentTransformations, texCoordGeneration, 
-              cullFace, backFaceNormalFlip);
+              textureTransform, cullFace, backFaceNormalFlip);
         }
       }
     }    
   }
   
+  /**
+   * Returns an input stream to read the given URL.
+   */
+  private InputStream openStream(URL url) throws IOException {
+    URLConnection connection = url.openConnection();
+    if (System.getProperty("os.name").startsWith("Windows")
+        && (connection instanceof JarURLConnection)) {
+      JarURLConnection urlConnection = (JarURLConnection)connection;
+      URL jarFileUrl = urlConnection.getJarFileURL();
+      if (jarFileUrl.getProtocol().equalsIgnoreCase("file")) {
+        try {
+          if (new File(jarFileUrl.toURI()).canWrite()) {
+            // Refuse to use caches to be able to delete the writable files accessed with jar protocol under Windows, 
+            // as suggested in http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6962459 
+            connection.setUseCaches(false);
+          }
+        } catch (URISyntaxException ex) {
+          IOException ex2 = new IOException();
+          ex2.initCause(ex);
+          throw ex2;
+        }
+      }
+    }
+    return connection.getInputStream();
+  }
+  
   /**
    * Returns <code>true</code> if <code>name</code> contains 
    * only letters, digits and underscores. 
@@ -454,6 +515,7 @@ public class OBJWriter extends FilterWriter {
   private void writeNodeGeometry(Geometry geometry, 
                                  Transform3D parentTransformations, 
                                  TexCoordGeneration texCoordGeneration, 
+                                 Transform3D textureTransform, 
                                  int cullFace, 
                                  boolean backFaceNormalFlip) throws IOException {
     if (geometry instanceof GeometryArray) {
@@ -519,14 +581,14 @@ public class OBJWriter extends FilterWriter {
                     index < n; index++, i += vertexSize) {
                 TexCoord2f textureCoordinates = generateTextureCoordinates(
                     vertexData [i], vertexData [i + 1], vertexData [i + 2], planeS, planeT);
-                writeTextureCoordinates(textureCoordinates, index, textureCoordinatesIndexSubstitutes);
+                writeTextureCoordinates(textureCoordinates, textureTransform, index, textureCoordinatesIndexSubstitutes);
               }
             }
           } else if (textureCoordinatesDefined) {
             for (int index = 0, i = 0, n = geometryArray.getVertexCount(); 
                   index < n; index++, i += vertexSize) {
               TexCoord2f textureCoordinates = new TexCoord2f(vertexData [i], vertexData [i + 1]);
-              writeTextureCoordinates(textureCoordinates, index, textureCoordinatesIndexSubstitutes);
+              writeTextureCoordinates(textureCoordinates, textureTransform, index, textureCoordinatesIndexSubstitutes);
             }
           }
           // Write normals
@@ -552,14 +614,14 @@ public class OBJWriter extends FilterWriter {
               for (int index = 0, i = 0, n = geometryArray.getVertexCount(); index < n; index++, i += 3) {
                 TexCoord2f textureCoordinates = generateTextureCoordinates(
                     vertexCoordinates [i], vertexCoordinates [i + 1], vertexCoordinates [i + 2], planeS, planeT);
-                writeTextureCoordinates(textureCoordinates, index, textureCoordinatesIndexSubstitutes);
+                writeTextureCoordinates(textureCoordinates, textureTransform, index, textureCoordinatesIndexSubstitutes);
               }
             }
           } else if (textureCoordinatesDefined) {
             float [] textureCoordinatesArray = geometryArray.getTexCoordRefFloat(0);
             for (int index = 0, i = 0, n = geometryArray.getVertexCount(); index < n; index++, i += 2) {
               TexCoord2f textureCoordinates = new TexCoord2f(textureCoordinatesArray [i], textureCoordinatesArray [i + 1]);
-              writeTextureCoordinates(textureCoordinates, index, textureCoordinatesIndexSubstitutes);
+              writeTextureCoordinates(textureCoordinates, textureTransform, index, textureCoordinatesIndexSubstitutes);
             }
           }
           // Write normals
@@ -588,14 +650,14 @@ public class OBJWriter extends FilterWriter {
               geometryArray.getCoordinate(index, vertex);
               TexCoord2f textureCoordinates = generateTextureCoordinates(
                   vertex.x, vertex.y, vertex.z, planeS, planeT);
-              writeTextureCoordinates(textureCoordinates, index, textureCoordinatesIndexSubstitutes);
+              writeTextureCoordinates(textureCoordinates, textureTransform, index, textureCoordinatesIndexSubstitutes);
             }
           }
         } else if (textureCoordinatesDefined) {
           for (int index = 0, n = geometryArray.getVertexCount(); index < n; index++) {
             TexCoord2f textureCoordinates = new TexCoord2f();
             geometryArray.getTextureCoordinate(0, index, textureCoordinates);
-            writeTextureCoordinates(textureCoordinates, index, textureCoordinatesIndexSubstitutes);
+            writeTextureCoordinates(textureCoordinates, textureTransform, index, textureCoordinatesIndexSubstitutes);
           }
         }
         // Write normals
@@ -845,8 +907,13 @@ public class OBJWriter extends FilterWriter {
    * Writes <code>textureCoordinates</code> in a line vt at OBJ format, 
    * if the texture coordinates wasn't written yet.  
    */
-  private void writeTextureCoordinates(TexCoord2f textureCoordinates, int index,
-                                       int [] textureCoordinatesIndexSubstitutes) throws IOException {
+  private void writeTextureCoordinates(TexCoord2f textureCoordinates, Transform3D textureTransform,
+                                       int index, int [] textureCoordinatesIndexSubstitutes) throws IOException {
+    if (textureTransform.getBestType() != Transform3D.IDENTITY) {
+      Point3f transformedCoordinates = new Point3f(textureCoordinates.x, textureCoordinates.y, 0);
+      textureTransform.transform(transformedCoordinates);
+      textureCoordinates = new TexCoord2f(transformedCoordinates.x, transformedCoordinates.y);
+    }
     Integer textureCoordinatesIndex = this.textureCoordinatesIndices.get(textureCoordinates);
     if (textureCoordinatesIndex == null) {
       textureCoordinatesIndexSubstitutes [index] = this.textureCoordinatesIndices.size() + 1;
@@ -897,8 +964,26 @@ public class OBJWriter extends FilterWriter {
       vertexIndex3 = tmp;
     }
     
-    if (textureCoordinatesGenerated
-        || (geometryArray.getVertexFormat() & GeometryArray.TEXTURE_COORDINATE_2) != 0) {
+    if (textureCoordinatesGenerated) {
+      if (normalsDefined) {
+        this.out.write("f " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
+            + "/" + (normalIndexSubstitutes [geometryArray.getNormalIndex(vertexIndex1)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
+            + "/" + (normalIndexSubstitutes [geometryArray.getNormalIndex(vertexIndex2)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex3)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex3)]) 
+            + "/" + (normalIndexSubstitutes [geometryArray.getNormalIndex(vertexIndex3)]) + "\n");
+      } else {
+        this.out.write("f " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex3)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex3)]) + "\n");
+      }
+    } else if ((geometryArray.getVertexFormat() & GeometryArray.TEXTURE_COORDINATE_2) != 0) {
       if (normalsDefined) {
         this.out.write("f " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
             + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getTextureCoordinateIndex(0, vertexIndex1)]) 
@@ -962,8 +1047,31 @@ public class OBJWriter extends FilterWriter {
       vertexIndex4 = tmp;
     }
     
-    if (textureCoordinatesGenerated
-        || (geometryArray.getVertexFormat() & GeometryArray.TEXTURE_COORDINATE_2) != 0) {
+    if (textureCoordinatesGenerated) {
+      if (normalsDefined) {
+        this.out.write("f " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
+            + "/" + (normalIndexSubstitutes [geometryArray.getNormalIndex(vertexIndex1)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
+            + "/" + (normalIndexSubstitutes [geometryArray.getNormalIndex(vertexIndex2)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex3)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex3)]) 
+            + "/" + (normalIndexSubstitutes [geometryArray.getNormalIndex(vertexIndex3)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex4)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex4)]) 
+            + "/" + (normalIndexSubstitutes [geometryArray.getNormalIndex(vertexIndex4)]) + "\n");
+      } else {
+        this.out.write("f " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex3)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex3)]) 
+            + " " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex4)]) 
+            + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex4)]) + "\n");
+      }
+    } else if ((geometryArray.getVertexFormat() & GeometryArray.TEXTURE_COORDINATE_2) != 0) {
       if (normalsDefined) {
         this.out.write("f " + (vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
             + "/" + (textureCoordinatesIndexSubstitutes [geometryArray.getTextureCoordinateIndex(0, vertexIndex1)]) 
@@ -1253,9 +1361,32 @@ public class OBJWriter extends FilterWriter {
       
       for (Map.Entry<Texture, File> textureEntry : this.textures.entrySet()) {
         Texture texture = textureEntry.getKey();
-        ImageComponent2D imageComponent = (ImageComponent2D)texture.getImage(0);
-        RenderedImage image = imageComponent.getRenderedImage();
-        ImageIO.write(image, "png", textureEntry.getValue());        
+        Object textureUrl = texture.getUserData();
+        if (this.copiedTextures.contains(textureUrl)) {
+          // Copy texture image file directly
+          InputStream in = null;
+          OutputStream out = null;
+          try {
+            in = openStream((URL)textureUrl);
+            out = new FileOutputStream(textureEntry.getValue());
+            byte [] buffer = new byte [8192];
+            int size; 
+            while ((size = in.read(buffer)) != -1) {
+              out.write(buffer, 0, size);
+            }
+          } finally {
+            if (in != null) {
+              in.close();
+            }
+            if (out != null) {
+              out.close();
+            }
+          }
+        } else {
+          ImageComponent2D imageComponent = (ImageComponent2D)texture.getImage(0);
+          RenderedImage image = imageComponent.getRenderedImage();
+          ImageIO.write(image, "png", textureEntry.getValue());
+        }
       }
     } finally {
       if (writer != null) {
@@ -1273,6 +1404,22 @@ public class OBJWriter extends FilterWriter {
                                         int compressionLevel,
                                         String entryName, 
                                         String header) throws IOException {
+    writeNodeInZIPFile(node, null, zipFile, compressionLevel, entryName, header);
+  }
+  
+  /**
+   * Writes <code>node</code> in an entry at OBJ format of the given zip file 
+   * along with its MTL file and texture images.
+   * Once saved, <code>materialAppearances</code> will contain the appearances matching 
+   * each material saved in the MTL file. Material names used as keys maybe be different     
+   * of the appearance names to respect MTL specifications.
+   */
+  public static void writeNodeInZIPFile(Node node, 
+                                        Map<String, Appearance> materialAppearances,
+                                        File zipFile,    
+                                        int compressionLevel,
+                                        String entryName, 
+                                        String header) throws IOException {
     // Create a temporary folder
     File tempFolder = null;
     for (int i = 0; i < 10 && tempFolder == null; i++) { 
@@ -1314,6 +1461,13 @@ public class OBJWriter extends FilterWriter {
           }          
         }
       }
+
+      if (materialAppearances != null) {
+        // Update material / appearance map
+        for (Map.Entry<ComparableAppearance, String> appearanceEntry : writer.appearances.entrySet()) {
+          materialAppearances.put(appearanceEntry.getValue(), appearanceEntry.getKey().getAppearance());
+        }
+      }
     } finally {
       if (zipOut != null) {
         zipOut.close();
