@@ -63,9 +63,9 @@ public abstract class Object3DBranch extends BranchGroup {
   protected static final Integer  DEFAULT_AMBIENT_COLOR = 0x333333;
   protected static final Material DEFAULT_MATERIAL      = new Material();
 
-  private static final Map<Long, Material>              materials = new HashMap<Long, Material>();
-  private static final Map<TextureKey, TextureAttributes>    textureAttributes = new HashMap<TextureKey, TextureAttributes>();
-  private static final Map<Home, Map<Texture, Texture>> homesTextures = new WeakHashMap<Home, Map<Texture, Texture>>();
+  private static final Map<Long, Material>                materials = new HashMap<Long, Material>();
+  private static final Map<TextureKey, TextureAttributes> textureAttributes = new HashMap<TextureKey, TextureAttributes>();
+  private static final Map<Home, Map<Texture, Texture>>   homesTextures = new WeakHashMap<Home, Map<Texture, Texture>>();
   
   static {
     DEFAULT_MATERIAL.setCapability(Material.ALLOW_COMPONENT_READ);
@@ -153,19 +153,32 @@ public abstract class Object3DBranch extends BranchGroup {
    * and scaled if required.
    */
   protected TextureAttributes getTextureAttributes(HomeTexture texture, boolean scaled) {
+    float textureWidth = texture.getWidth();
+    float textureHeight = texture.getHeight();
+    if (textureWidth == -1 || textureHeight == -1) {
+      // Set a default value of 1m for textures with width and height equal to -1
+      // (this may happen for textures retrieved from 3D models)
+      textureWidth = 100;
+      textureHeight = 100;
+    }
+    float textureAngle = texture.getAngle();
+    float textureScale = 1 / texture.getScale();
     TextureKey key = scaled
-        ? new TextureKey(texture.getWidth(), texture.getHeight(), texture.getAngle())
-        : new TextureKey(-1f, -1f, texture.getAngle());
+        ? new TextureKey(textureWidth, textureHeight, textureAngle, textureScale)
+        : new TextureKey(-1f, -1f, textureAngle, textureScale);
     TextureAttributes textureAttributes = Object3DBranch.textureAttributes.get(key);
     if (textureAttributes == null) {
       textureAttributes = new TextureAttributes();
       // Mix texture and color
       textureAttributes.setTextureMode(TextureAttributes.MODULATE);
       Transform3D rotation = new Transform3D();
-      rotation.rotZ(texture.getAngle());
+      rotation.rotZ(textureAngle);
       Transform3D transform = new Transform3D();
+      // Change scale if required
       if (scaled) {
-        transform.setScale(new Vector3d(1. / texture.getWidth(), 1. / texture.getHeight(), 1));
+        transform.setScale(new Vector3d(textureScale / textureWidth, textureScale / textureHeight, textureScale));
+      } else {
+        transform.setScale(textureScale);
       }
       transform.mul(rotation);
       textureAttributes.setTextureTransform(transform);
@@ -182,11 +195,13 @@ public abstract class Object3DBranch extends BranchGroup {
     private final float width;
     private final float height;
     private final float angle;
+    private final float scale;
     
-    public TextureKey(float width, float height, float angle) {
+    public TextureKey(float width, float height, float angle, float scale) {
       this.width = width;
       this.height = height;
       this.angle = angle;
+      this.scale = scale;
     }
     
     @Override
@@ -194,14 +209,16 @@ public abstract class Object3DBranch extends BranchGroup {
       TextureKey key = (TextureKey)obj;
       return this.width == key.width 
           && this.height == key.height 
-          && this.angle == key.angle;
+          && this.angle == key.angle 
+          && this.scale == key.scale;
     }
     
     @Override
     public int hashCode() {
       return Float.floatToIntBits(this.width) * 31 
           + Float.floatToIntBits(this.height) * 31
-          + Float.floatToIntBits(this.angle);
+          + Float.floatToIntBits(this.angle) * 31
+          + Float.floatToIntBits(this.scale);
     }
   }
 
