@@ -20,6 +20,7 @@
 package com.eteks.sweethome3d.model;
 
 import java.math.BigDecimal;
+import java.util.Map;
 
 /**
  * A light of the catalog.
@@ -355,14 +356,68 @@ public class CatalogLight extends CatalogPieceOfFurniture implements Light {
                       float [][] modelRotation, boolean backFaceShown, Long modelSize, String creator, 
                       boolean resizable, boolean deformable, boolean texturable, boolean horizontallyRotatable,
                       BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency) {
+    this(id, name, description, information, tags, creationDate, grade, 
+        icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable, lightSources, 
+        staircaseCutOutShape, modelRotation, backFaceShown, modelSize, creator, resizable, deformable, texturable, horizontallyRotatable,
+        price, valueAddedTaxPercentage, currency, null);
+  }
+  
+  /**
+   * Creates an unmodifiable catalog light of the default catalog.
+   * @param id    the id of the new light, or <code>null</code>
+   * @param name  the name of the new light
+   * @param description the description of the new light 
+   * @param information additional information associated to the new light
+   * @param tags tags associated to the new light
+   * @param creationDate creation date of the new light in milliseconds since the epoch 
+   * @param grade grade of the new light or <code>null</code>
+   * @param icon content of the icon of the new light
+   * @param planIcon content of the icon of the new piece displayed in plan
+   * @param model content of the 3D model of the new light
+   * @param width  the width in centimeters of the new light
+   * @param depth  the depth in centimeters of the new light
+   * @param height  the height in centimeters of the new light
+   * @param dropOnTopElevation a percentage of the height at which should be placed 
+   *            an object dropped on the new piece
+   * @param elevation  the elevation in centimeters of the new light
+   * @param movable if <code>true</code>, the new light is movable
+   * @param lightSources the light sources of the new light
+   * @param staircaseCutOutShape the shape used to cut out upper levels when they intersect 
+   *            with the piece like a staircase
+   * @param modelRotation the rotation 3 by 3 matrix applied to the light model
+   * @param backFaceShown <code>true</code> if back face should be shown instead of front faces
+   * @param modelSize size of the 3D model of the new light
+   * @param creator the creator of the model
+   * @param resizable if <code>true</code>, the size of the new light may be edited
+   * @param deformable if <code>true</code>, the width, depth and height of the new piece may 
+   *            change independently from each other
+   * @param texturable if <code>false</code> this piece should always keep the same color or texture
+   * @param horizontallyRotatable if <code>false</code> this piece 
+   *            should not rotate around an horizontal axis
+   * @param price the price of the new light, or <code>null</code> 
+   * @param valueAddedTaxPercentage the Value Added Tax percentage applied to the 
+   *             price of the new light or <code>null</code> 
+   * @param currency the price currency, noted with ISO 4217 code, or <code>null</code> 
+   * @param properties additional properties associating a key to a value or <code>null</code>
+   * @since 5.7          
+   */
+  public CatalogLight(String id, String name, String description, 
+                      String information, String [] tags, Long creationDate, Float grade, 
+                      Content icon, Content planIcon, Content model, 
+                      float width, float depth, float height, float elevation, float dropOnTopElevation, 
+                      boolean movable, LightSource [] lightSources, String staircaseCutOutShape, 
+                      float [][] modelRotation, boolean backFaceShown, Long modelSize, String creator, 
+                      boolean resizable, boolean deformable, boolean texturable, boolean horizontallyRotatable,
+                      BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency,
+                      Map<String, String> properties) {
     super(id, name, description, information, tags, creationDate, grade, 
         icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable, 
         staircaseCutOutShape, modelRotation, backFaceShown, modelSize, creator, 
         resizable, deformable, texturable, horizontallyRotatable, 
-        price, valueAddedTaxPercentage, currency);
+        price, valueAddedTaxPercentage, currency, properties);
     this.lightSources = lightSources;
   }
-         
+      
   /**
    * Returns the sources managed by this light. Each light source point
    * is a percentage of the width, the depth and the height of this light,
