@@ -82,10 +82,10 @@ public class HomeEnvironment implements Serializable, Cloneable {
    */
   public HomeEnvironment() {
     this(0xA8A8A8, // Ground color
-        null,      // Ground texture
-        (204 << 16) + (228 << 8) + 252, // Sky color
-        0xD0D0D0,  // Light color
-        0);        // Walls alpha
+         null,     // Ground texture
+         0xCCE4FC, // Sky color
+         0xD0D0D0, // Light color
+         0);       // Walls alpha
   }
 
   /**
@@ -376,14 +376,14 @@ public class HomeEnvironment implements Serializable, Cloneable {
   }
 
   /**
-   * Returns whether all levels should be displayed or not.
+   * Returns whether all levels should be visible or not.
    */
   public boolean isAllLevelsVisible() {
     return this.allLevelsVisible;
   }
   
   /**
-   * Sets whether all levels should be displayed or not and fires a <code>PropertyChangeEvent</code>.
+   * Sets whether all levels should be visible or not and fires a <code>PropertyChangeEvent</code>.
    */
   public void setAllLevelsVisible(boolean allLevelsVisible) {
     if (allLevelsVisible != this.allLevelsVisible) {
