@@ -33,7 +33,6 @@ import java.awt.RenderingHints;
 import java.awt.dnd.DnDConstants;
 import java.awt.event.AdjustmentListener;
 import java.awt.event.MouseEvent;
-import java.awt.event.MouseMotionAdapter;
 import java.lang.ref.WeakReference;
 import java.net.MalformedURLException;
 import java.net.URL;
@@ -111,6 +110,11 @@ public class FurnitureCatalogTree extends JTree implements View {
                               UserPreferences preferences, 
                               FurnitureCatalogController controller) {
     this.preferences = preferences;
+    float resolutionScale = SwingTools.getResolutionScale();
+    if (resolutionScale != 1) {
+      // Adapt row height to specified resolution scale
+      setRowHeight(Math.round(getRowHeight() * resolutionScale));
+    }
     this.toolTip = new CatalogItemToolTip(true, preferences);
     setModel(new CatalogTreeModel(catalog));
     setRootVisible(false);
@@ -233,8 +237,8 @@ public class FurnitureCatalogTree extends JTree implements View {
                   && clickedPath.getLastPathComponent() instanceof CatalogPieceOfFurniture) {
                 controller.modifySelectedFurniture();
               }
-            } else if (getCellRenderer() instanceof CatalogCellRenderer) {
-              URL url = ((CatalogCellRenderer)getCellRenderer()).getURLAt(ev.getPoint(), (JTree)ev.getSource());
+            } else {
+              URL url = getURLAt(ev.getPoint(), (JTree)ev.getSource());
               if (url != null) {
                 SwingTools.showDocumentInBrowser(url);
               }
@@ -244,17 +248,57 @@ public class FurnitureCatalogTree extends JTree implements View {
         
         @Override
         public void mouseMoved(MouseEvent ev) {
-          if (getCellRenderer() instanceof CatalogCellRenderer) {
-            URL url = ((CatalogCellRenderer)getCellRenderer()).getURLAt(ev.getPoint(), (JTree)ev.getSource());
-            if (url != null) {
-              EventQueue.invokeLater(new Runnable() {                  
-                  public void run() {
-                    setCursor(handCursor);
+          final URL url = getURLAt(ev.getPoint(), (JTree)ev.getSource());
+          EventQueue.invokeLater(new Runnable() {                  
+              public void run() {
+                if (url != null) {
+                  setCursor(handCursor);
+                } else {
+                  setCursor(Cursor.getDefaultCursor());
+                }
+              }
+            });
+        }
+
+        private URL getURLAt(Point point, JTree tree) {
+          TreePath path = tree.getPathForLocation(point.x, point.y);
+          if (path != null
+              && path.getLastPathComponent() instanceof CatalogPieceOfFurniture) {
+            CatalogPieceOfFurniture piece = (CatalogPieceOfFurniture)path.getLastPathComponent();
+            String information = piece.getInformation();
+            if (information != null) {
+              int row = tree.getRowForPath(path);
+              JComponent rendererComponent = (JComponent)tree.getCellRenderer().
+                  getTreeCellRendererComponent(tree, piece, false, false, false, row, false);
+              rendererComponent.doLayout();
+              for (JEditorPane pane : SwingTools.findChildren(rendererComponent, JEditorPane.class)) {
+                Rectangle rowBounds = tree.getRowBounds(row);
+                point.x -= rowBounds.x + pane.getX(); 
+                point.y -= rowBounds.y + pane.getY(); 
+                if (point.x > 0 && point.y > 0) {
+                  // Search in information pane if point is over a HTML link
+                  int position = pane.viewToModel(point);
+                  if (position > 0) {
+                    HTMLDocument hdoc = (HTMLDocument)pane.getDocument();
+                    Element element = hdoc.getCharacterElement(position);
+                    AttributeSet a = element.getAttributes();
+                    AttributeSet anchor = (AttributeSet)a.getAttribute(HTML.Tag.A);
+                    if (anchor != null) {
+                      String href = (String)anchor.getAttribute(HTML.Attribute.HREF);
+                      if (href != null) {
+                        try {
+                          return new URL(href);
+                        } catch (MalformedURLException ex) {
+                          // Ignore malformed URL
+                        }
+                      }
+                    }
                   }
-                });
+                }
+              }
             }
           }
-          setCursor(Cursor.getDefaultCursor());
+          return null;
         }
       };
     addMouseListener(mouseListener);
@@ -351,7 +395,7 @@ public class FurnitureCatalogTree extends JTree implements View {
         this.defaultFont = this.nameLabel.getFont();
         String bodyRule = "body { font-family: " + this.defaultFont.getFamily() + "; " 
             + "font-size: " + this.defaultFont.getSize() + "pt; " 
-            + "top-margin: 0; }";
+            + "margin: 0; }";
         ((HTMLDocument)this.informationPane.getDocument()).getStyleSheet().addRule(bodyRule);
         this.modifiablePieceFont = 
             new Font(this.defaultFont.getFontName(), Font.ITALIC, this.defaultFont.getSize());        
@@ -370,13 +414,8 @@ public class FurnitureCatalogTree extends JTree implements View {
         this.nameLabel.setFont(piece.isModifiable() 
             ? this.modifiablePieceFont : this.defaultFont);
         
-        String information = piece.getInformation();
-        if (information != null) {
-          this.informationPane.setText(information);
-          this.informationPane.setVisible(true);
-        } else {
-          this.informationPane.setVisible(false);
-        }
+        this.informationPane.setVisible(true);
+        this.informationPane.setText(piece.getInformation());
       }
       return this;
     }
@@ -388,7 +427,7 @@ public class FurnitureCatalogTree extends JTree implements View {
       if (this.informationPane.isVisible()) {
         Dimension informationPreferredSize = this.informationPane.getPreferredSize();
         this.informationPane.setBounds(namePreferredSize.width + 2, 
-            (namePreferredSize.height - informationPreferredSize.height) / 2,
+            Math.max(0, (namePreferredSize.height - informationPreferredSize.height) / 2),
             informationPreferredSize.width, namePreferredSize.height);
       }
     }
@@ -446,43 +485,6 @@ public class FurnitureCatalogTree extends JTree implements View {
           RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
       super.paintChildren(g);
     }
-
-    public URL getURLAt(Point point, JTree tree) {
-      TreePath path = tree.getPathForLocation(point.x, point.y);
-      if (path != null
-          && path.getLastPathComponent() instanceof CatalogPieceOfFurniture) {
-        CatalogPieceOfFurniture piece = (CatalogPieceOfFurniture)path.getLastPathComponent();
-        String information = piece.getInformation();
-        if (information != null) {
-          int row = tree.getRowForPath(path);
-          getTreeCellRendererComponent(tree, piece, false, false, false, row, false).doLayout();
-          Rectangle rowBounds = tree.getRowBounds(row);
-          point.x -= rowBounds.x + this.informationPane.getX(); 
-          point.y -= rowBounds.y + this.informationPane.getY(); 
-          if (point.x > 0 && point.y > 0) {
-            // Search in information pane if point is over a HTML link
-            int position = this.informationPane.viewToModel(point);
-            if (position > 0) {
-              HTMLDocument hdoc = (HTMLDocument)this.informationPane.getDocument();
-              Element element = hdoc.getCharacterElement(position);
-              AttributeSet a = element.getAttributes();
-              AttributeSet anchor = (AttributeSet)a.getAttribute(HTML.Tag.A);
-              if (anchor != null) {
-                String href = (String)anchor.getAttribute(HTML.Attribute.HREF);
-                if (href != null) {
-                  try {
-                    return new URL(href);
-                  } catch (MalformedURLException ex) {
-                    // Ignore malformed URL
-                  }
-                }
-              }
-            }
-          }
-        }
-      }
-      return null;
-    }
   }
   
   /**
@@ -513,9 +515,13 @@ public class FurnitureCatalogTree extends JTree implements View {
     public int getChildCount(Object parent) {
       if (parent instanceof FurnitureCatalog) {
         return ((FurnitureCatalog)parent).getCategoriesCount();
-      } else {
+      } else if (parent instanceof FurnitureCategory) {
         return ((FurnitureCategory)parent).getFurnitureCount();
-      } 
+      } else {
+        // Shouldn't be necessary for other tree items, but javax.swing.plaf.basic.BasicTreeUI$Actions.traverse 
+        // might call getChildCount even for tree leaves   
+        return 0;
+      }
     }
 
     public int getIndexOfChild(Object parent, Object child) {
