@@ -103,7 +103,7 @@ public class Home3DAttributesPanel extends JPanel implements DialogView {
           }
         });
   
-    this.groundColorButton = new ColorButton();
+    this.groundColorButton = new ColorButton(preferences);
     this.groundColorButton.setColorDialogTitle(preferences.getLocalizedString(
         Home3DAttributesPanel.class, "groundColorDialog.title"));
     this.groundColorButton.setColor(controller.getGroundColor());
@@ -154,7 +154,7 @@ public class Home3DAttributesPanel extends JPanel implements DialogView {
           }
         });
   
-    this.skyColorButton = new ColorButton();
+    this.skyColorButton = new ColorButton(preferences);
     this.skyColorButton.setColorDialogTitle(preferences.getLocalizedString(
         Home3DAttributesPanel.class, "skyColorDialog.title"));
     this.skyColorButton.setColor(controller.getSkyColor());
@@ -234,7 +234,7 @@ public class Home3DAttributesPanel extends JPanel implements DialogView {
             wallsTransparencySlider.setValue((int)(controller.getWallsAlpha() * 255));
           }
         });
-
+    
     this.dialogTitle = preferences.getLocalizedString(
         Home3DAttributesPanel.class, "home3DAttributes.title");
   }
@@ -371,7 +371,7 @@ public class Home3DAttributesPanel extends JPanel implements DialogView {
     wallsTransparencyLabelsPanel.add(this.invisibleWallsTransparencyLabel, BorderLayout.EAST);
     renderingPanel.add(wallsTransparencyLabelsPanel, new GridBagConstraints(
         1, 3, 3, 1, 1, 0, GridBagConstraints.CENTER, 
-        GridBagConstraints.HORIZONTAL, new Insets(OperatingSystem.isWindows() ? 0 : -3, 0, 0, 0), 0, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(OperatingSystem.isWindows() ? 0 : -3, 0, 10, 0), 0, 0));
     add(renderingPanel, new GridBagConstraints(
         0, 2, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
