@@ -99,6 +99,8 @@ import com.sun.j3d.utils.universe.ViewingPlatform;
  * Super class of 3D preview component for model. 
  */
 public class ModelPreviewComponent extends JComponent {
+  private static final int MODEL_PREFERRED_SIZE = Math.round(200 * SwingTools.getResolutionScale());
+  
   private SimpleUniverse          universe;
   private JPanel                  component3DPanel;
   private Component               component3D;
@@ -163,7 +165,7 @@ public class ModelPreviewComponent extends JComponent {
     if (isPreferredSizeSet()) {
       return super.getPreferredSize();
     } else {
-      return new Dimension(200, 200);
+      return new Dimension(MODEL_PREFERRED_SIZE, MODEL_PREFERRED_SIZE);
     }
   }
 
@@ -545,21 +547,18 @@ public class ModelPreviewComponent extends JComponent {
     double nominalDistanceToCenter = 1.4 / Math.tan(Math.PI / 8);
     // We don't use a TransformGroup in scene tree to be able to share the same scene 
     // in the different views displayed by OrientationPreviewComponent class 
+    Transform3D translation = new Transform3D();
+    translation.setTranslation(new Vector3d(0, 0, nominalDistanceToCenter));
     Transform3D pitchRotation = new Transform3D();
     pitchRotation.rotX(viewPitch);
     Transform3D yawRotation = new Transform3D();
     yawRotation.rotY(viewYaw);
-    Transform3D transform = new Transform3D();
-    transform.setTranslation(
-        new Vector3d(Math.sin(viewYaw) * nominalDistanceToCenter * Math.cos(viewPitch), 
-            nominalDistanceToCenter * Math.sin(-viewPitch), 
-            Math.cos(viewYaw) * nominalDistanceToCenter * Math.cos(viewPitch)));
     Transform3D scale = new Transform3D();
     scale.setScale(viewScale);
     
+    pitchRotation.mul(translation);
     yawRotation.mul(pitchRotation);
-    transform.mul(yawRotation);
-    scale.mul(transform);
+    scale.mul(yawRotation);
     viewPlatformTransform.setTransform(scale);
   }
   
@@ -632,7 +631,9 @@ public class ModelPreviewComponent extends JComponent {
    * Returns the 3D model content displayed by this component.
    */
   public Content getModel() {
-    return this.previewedPiece.getModel();
+    return this.previewedPiece != null
+        ? this.previewedPiece.getModel()
+        : null;
   }
 
   /**
@@ -877,7 +878,7 @@ public class ModelPreviewComponent extends JComponent {
             this.iconImageLock.wait(maxWaitingDelay / 2);
             if (OperatingSystem.isMacOSX()) {
               // Under Mac OS X, sleep an additional time to ensure the screen got refreshed
-              Thread.sleep(25);
+              Thread.sleep(30);
             }
           } catch (InterruptedException ex) {
             ex.printStackTrace();
@@ -895,7 +896,7 @@ public class ModelPreviewComponent extends JComponent {
             this.iconImageLock.wait(maxWaitingDelay / 2);
             if (OperatingSystem.isMacOSX()) {
               // Under Mac OS X, sleep an additional time to ensure the screen got refreshed
-              Thread.sleep(25);
+              Thread.sleep(30);
             }
           } catch (InterruptedException ex) {
             ex.printStackTrace();
