@@ -96,6 +96,7 @@ import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.viewcontroller.AbstractPhotoController;
 import com.eteks.sweethome3d.viewcontroller.ContentManager;
 import com.eteks.sweethome3d.viewcontroller.DialogView;
+import com.eteks.sweethome3d.viewcontroller.Object3DFactory;
 import com.eteks.sweethome3d.viewcontroller.PhotoController;
 import com.eteks.sweethome3d.viewcontroller.View;
 
@@ -116,6 +117,7 @@ public class PhotoPanel extends JPanel implements DialogView {
   
   private final Home               home;
   private final UserPreferences    preferences;
+  private final Object3DFactory    object3dFactory;
   private final PhotoController    controller;
   private ScaledImageComponent     photoComponent; 
   private JLabel                   animatedWaitLabel;
@@ -143,9 +145,17 @@ public class PhotoPanel extends JPanel implements DialogView {
   public PhotoPanel(Home home, 
                     UserPreferences preferences, 
                     PhotoController controller) {
+    this(home, preferences, null, controller);
+  }
+  
+  public PhotoPanel(Home home, 
+                    UserPreferences preferences,
+                    Object3DFactory object3dFactory, 
+                    PhotoController controller) {
     super(new GridBagLayout());
     this.home = home;
     this.preferences = preferences;
+    this.object3dFactory = object3dFactory;
     this.controller = controller;
     createActions(preferences);
     createComponents(home, preferences, controller);
@@ -355,8 +365,8 @@ public class PhotoPanel extends JPanel implements DialogView {
     timeSpinnerModel.addChangeListener(dateTimeChangeListener);
 
     this.dayNightLabel = new JLabel();
-    final ImageIcon dayIcon = new ImageIcon(PhotoPanel.class.getResource("resources/day.png"));
-    final ImageIcon nightIcon = new ImageIcon(PhotoPanel.class.getResource("resources/night.png"));
+    final ImageIcon dayIcon = SwingTools.getScaledImageIcon(PhotoPanel.class.getResource("resources/day.png"));
+    final ImageIcon nightIcon = SwingTools.getScaledImageIcon(PhotoPanel.class.getResource("resources/night.png"));
     PropertyChangeListener dayNightListener = new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent ev) {
           if (home.getCompass().getSunElevation(
@@ -581,6 +591,7 @@ public class PhotoPanel extends JPanel implements DialogView {
       boolean advancedComponentsVisible = this.advancedComponentsSeparator.isVisible();
       if (advancedComponentsVisible != highQuality) {
         setAdvancedComponentsVisible(highQuality);
+        updateRatioComponents();
         int componentsHeight = this.advancedComponentsSeparator.getPreferredSize().height + 6
             + this.dateSpinner.getPreferredSize().height + 5
             + this.lensComboBox.getPreferredSize().height;
@@ -607,8 +618,9 @@ public class PhotoPanel extends JPanel implements DialogView {
    */
   private void updateRatioComponents() {
     Lens lens = this.controller.getLens();
-    boolean fixedProportions = lens == Camera.Lens.FISHEYE 
-        || lens == Camera.Lens.SPHERICAL;
+    boolean fixedProportions = this.lensComboBox.isVisible()
+        && (lens == Camera.Lens.FISHEYE 
+            || lens == Camera.Lens.SPHERICAL);
     this.sizeAndQualityPanel.setProportionsChoiceEnabled(!fixedProportions);
   }
 
@@ -635,8 +647,8 @@ public class PhotoPanel extends JPanel implements DialogView {
       Point dialogLocation = null;
       if (homeRoot != null) {
         // Restore location if it exists
-        Integer x = (Integer)this.home.getVisualProperty(PHOTO_DIALOG_X_VISUAL_PROPERTY);
-        Integer y = (Integer)this.home.getVisualProperty(PHOTO_DIALOG_Y_VISUAL_PROPERTY);      
+        Number x = this.home.getNumericProperty(PHOTO_DIALOG_X_VISUAL_PROPERTY);
+        Number y = this.home.getNumericProperty(PHOTO_DIALOG_Y_VISUAL_PROPERTY);      
 
         int windowRightBorder = homeRoot.getX() + homeRoot.getWidth();
         Dimension screenSize = getToolkit().getScreenSize();
@@ -655,9 +667,9 @@ public class PhotoPanel extends JPanel implements DialogView {
         }
         int dialogHeight = dialog.getHeight();
         if (x != null && y != null 
-            && x + dialogWidth <= screenRightBorder
-            && y + dialogHeight <= screenBottomBorder) {
-          dialogLocation = new Point(x, y);
+            && x.intValue() + dialogWidth <= screenRightBorder
+            && y.intValue() + dialogHeight <= screenBottomBorder) {
+          dialogLocation = new Point(x.intValue(), y.intValue());
         } else if (screenRightBorder - windowRightBorder > dialogWidth / 2
                    || dialogHeight == screenHeight) {
           // If there some space left at the right of the window,
@@ -694,8 +706,8 @@ public class PhotoPanel extends JPanel implements DialogView {
 
           @Override
           public void componentMoved(ComponentEvent ev) {
-            controller.setVisualProperty(PHOTO_DIALOG_X_VISUAL_PROPERTY, dialog.getX());
-            controller.setVisualProperty(PHOTO_DIALOG_Y_VISUAL_PROPERTY, dialog.getY());
+            controller.setHomeProperty(PHOTO_DIALOG_X_VISUAL_PROPERTY, String.valueOf(dialog.getX()));
+            controller.setHomeProperty(PHOTO_DIALOG_Y_VISUAL_PROPERTY, String.valueOf(dialog.getY()));
           }
         });
       dialog.setVisible(true);
@@ -744,7 +756,7 @@ public class PhotoPanel extends JPanel implements DialogView {
       int imageHeight = this.controller.getHeight();
       if (quality >= 2) {
         // Use photo renderer
-        PhotoRenderer photoRenderer = new PhotoRenderer(home, 
+        PhotoRenderer photoRenderer = new PhotoRenderer(home, this.object3dFactory, 
             quality == 2 
                 ? PhotoRenderer.Quality.LOW 
                 : PhotoRenderer.Quality.HIGH);
@@ -771,7 +783,8 @@ public class PhotoPanel extends JPanel implements DialogView {
         }
       } else {
         // Compute 3D view offscreen image
-        HomeComponent3D homeComponent3D = new HomeComponent3D(home, this.preferences, quality == 1);
+        HomeComponent3D homeComponent3D = new HomeComponent3D(
+            home, this.preferences, this.object3dFactory, quality == 1, null);
         image = homeComponent3D.getOffScreenImage(imageWidth, imageHeight);
       }
     } catch (OutOfMemoryError ex) {
