@@ -93,6 +93,7 @@ import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.viewcontroller.ContentManager;
 import com.eteks.sweethome3d.viewcontroller.DialogView;
+import com.eteks.sweethome3d.viewcontroller.Object3DFactory;
 import com.eteks.sweethome3d.viewcontroller.PhotosController;
 import com.eteks.sweethome3d.viewcontroller.View;
 
@@ -112,6 +113,7 @@ public class PhotosPanel extends JPanel implements DialogView {
   
   private final Home               home;
   private final UserPreferences    preferences;
+  private final Object3DFactory    object3dFactory;
   private final PhotosController   controller;
   private JLabel                   selectedCamerasLabel;
   private JList                    selectedCamerasList;
@@ -135,9 +137,17 @@ public class PhotosPanel extends JPanel implements DialogView {
   public PhotosPanel(Home home, 
                      UserPreferences preferences, 
                      PhotosController controller) {
+    this(home, preferences, null, controller);
+  }
+  
+  public PhotosPanel(Home home, 
+                     UserPreferences preferences, 
+                     Object3DFactory object3dFactory,
+                     PhotosController controller) {
     super(new GridBagLayout());
     this.home = home;
     this.preferences = preferences;
+    this.object3dFactory = object3dFactory;
     this.controller = controller;
     createActions(preferences);
     createComponents(home, preferences, controller);
@@ -476,8 +486,8 @@ public class PhotosPanel extends JPanel implements DialogView {
       Point dialogLocation = null;
       if (homeRoot != null) {
         // Restore location if it exists
-        Integer x = (Integer)this.home.getVisualProperty(PHOTOS_DIALOG_X_VISUAL_PROPERTY);
-        Integer y = (Integer)this.home.getVisualProperty(PHOTOS_DIALOG_Y_VISUAL_PROPERTY);      
+        Number x = this.home.getNumericProperty(PHOTOS_DIALOG_X_VISUAL_PROPERTY);
+        Number y = this.home.getNumericProperty(PHOTOS_DIALOG_Y_VISUAL_PROPERTY);      
 
         int windowRightBorder = homeRoot.getX() + homeRoot.getWidth();
         Dimension screenSize = getToolkit().getScreenSize();
@@ -496,9 +506,9 @@ public class PhotosPanel extends JPanel implements DialogView {
         }
         int dialogHeight = dialog.getHeight();
         if (x != null && y != null 
-            && x + dialogWidth <= screenRightBorder
-            && y + dialogHeight <= screenBottomBorder) {
-          dialogLocation = new Point(x, y);
+            && x.intValue() + dialogWidth <= screenRightBorder
+            && y.intValue() + dialogHeight <= screenBottomBorder) {
+          dialogLocation = new Point(x.intValue(), y.intValue());
         } else if (screenRightBorder - windowRightBorder > dialogWidth / 2
                    || dialogHeight == screenHeight) {
           // If there some space left at the right of the window,
@@ -533,8 +543,8 @@ public class PhotosPanel extends JPanel implements DialogView {
 
           @Override
           public void componentMoved(ComponentEvent ev) {
-            controller.setVisualProperty(PHOTOS_DIALOG_X_VISUAL_PROPERTY, dialog.getX());
-            controller.setVisualProperty(PHOTOS_DIALOG_Y_VISUAL_PROPERTY, dialog.getY());
+            controller.setHomeProperty(PHOTOS_DIALOG_X_VISUAL_PROPERTY, String.valueOf(dialog.getX()));
+            controller.setHomeProperty(PHOTOS_DIALOG_Y_VISUAL_PROPERTY, String.valueOf(dialog.getY()));
           }
         });
       dialog.setVisible(true);
@@ -632,7 +642,7 @@ public class PhotosPanel extends JPanel implements DialogView {
         home.setCamera(camera);
         if (quality >= 2) {
           // Use photo renderer
-          PhotoRenderer photoRenderer = new PhotoRenderer(home, 
+          PhotoRenderer photoRenderer = new PhotoRenderer(home, this.object3dFactory,
             quality == 2 
                 ? PhotoRenderer.Quality.LOW 
                 : PhotoRenderer.Quality.HIGH);
@@ -653,7 +663,8 @@ public class PhotosPanel extends JPanel implements DialogView {
           }
         } else {
           // Compute 3D view offscreen image
-          HomeComponent3D homeComponent3D = new HomeComponent3D(home, this.preferences, quality == 1);
+          HomeComponent3D homeComponent3D = new HomeComponent3D(
+              home, this.preferences, this.object3dFactory, quality == 1, null);
           updateProgressBar(photoIndex++, cameraFiles.size());
           image = homeComponent3D.getOffScreenImage(imageWidth, imageHeight);
           this.photoComponent.setImage(image);
