@@ -26,7 +26,6 @@ import java.lang.ref.WeakReference;
 
 import javax.swing.AbstractAction;
 import javax.swing.Action;
-import javax.swing.ImageIcon;
 import javax.swing.KeyStroke;
 import javax.swing.event.SwingPropertyChangeSupport;
 
@@ -121,7 +120,7 @@ public class ResourceAction extends AbstractAction {
     
     String smallIcon = getOptionalString(preferences, resourceClass, propertyPrefix + SMALL_ICON, false);
     if (smallIcon != null) {
-      putValue(SMALL_ICON, new ImageIcon(resourceClass.getResource(smallIcon)));
+      putValue(SMALL_ICON, SwingTools.getScaledImageIcon(resourceClass.getResource(smallIcon)));
     }
 
     String propertyKey = propertyPrefix + ACCELERATOR_KEY;
