@@ -19,24 +19,32 @@
  */
 package com.eteks.sweethome3d.swing;
 
+import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Component;
+import java.awt.Container;
 import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.EventQueue;
+import java.awt.Font;
 import java.awt.Frame;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
 import java.awt.GraphicsDevice;
 import java.awt.GraphicsEnvironment;
 import java.awt.GridBagLayout;
+import java.awt.Image;
 import java.awt.Insets;
+import java.awt.MouseInfo;
 import java.awt.Point;
 import java.awt.Rectangle;
+import java.awt.Stroke;
 import java.awt.Toolkit;
 import java.awt.Window;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.awt.event.AdjustmentEvent;
+import java.awt.event.AdjustmentListener;
 import java.awt.event.ComponentAdapter;
 import java.awt.event.ComponentEvent;
 import java.awt.event.FocusEvent;
@@ -54,6 +62,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
@@ -62,13 +71,17 @@ import java.util.ResourceBundle;
 import java.util.concurrent.Executors;
 
 import javax.imageio.ImageIO;
+import javax.imageio.ImageReader;
+import javax.imageio.stream.ImageInputStream;
 import javax.jnlp.BasicService;
 import javax.jnlp.ServiceManager;
 import javax.jnlp.UnavailableServiceException;
 import javax.swing.BorderFactory;
 import javax.swing.ButtonGroup;
+import javax.swing.ImageIcon;
 import javax.swing.JComponent;
 import javax.swing.JDialog;
+import javax.swing.JFrame;
 import javax.swing.JMenu;
 import javax.swing.JMenuItem;
 import javax.swing.JOptionPane;
@@ -78,9 +91,11 @@ import javax.swing.JRadioButton;
 import javax.swing.JScrollPane;
 import javax.swing.JSeparator;
 import javax.swing.JToggleButton;
+import javax.swing.JToolTip;
 import javax.swing.JViewport;
 import javax.swing.SwingUtilities;
 import javax.swing.Timer;
+import javax.swing.ToolTipManager;
 import javax.swing.UIManager;
 import javax.swing.border.AbstractBorder;
 import javax.swing.border.Border;
@@ -88,6 +103,8 @@ import javax.swing.event.PopupMenuEvent;
 import javax.swing.event.PopupMenuListener;
 import javax.swing.text.JTextComponent;
 
+import com.eteks.sweethome3d.model.Content;
+import com.eteks.sweethome3d.model.Polyline;
 import com.eteks.sweethome3d.model.TextureImage;
 import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.tools.OperatingSystem;
@@ -336,7 +353,8 @@ public class SwingTools {
     if (OperatingSystem.isMacOSX()
         && (language.equals(Locale.CHINESE.getLanguage())
             || language.equals(Locale.JAPANESE.getLanguage())
-            || language.equals(Locale.KOREAN.getLanguage()))) {
+            || language.equals(Locale.KOREAN.getLanguage())
+            || language.equals("uk"))) {  // Ukrainian
       int openingBracketIndex = localizedString.indexOf('(');
       if (openingBracketIndex != -1) {
         int closingBracketIndex = localizedString.indexOf(')');
@@ -445,7 +463,7 @@ public class SwingTools {
       dialog.addComponentListener(new ComponentAdapter() {
           @Override
           public void componentShown(ComponentEvent ev) {
-            requestFocus(focusedComponent);
+            requestFocusInWindow(focusedComponent);
             dialog.removeComponentListener(this);
           }
         });
@@ -464,7 +482,7 @@ public class SwingTools {
   /**
    * Requests the focus for the given component.
    */
-  private static void requestFocus(final JComponent focusedComponent) {
+  public static void requestFocusInWindow(final JComponent focusedComponent) {
     if (!focusedComponent.requestFocusInWindow()) {
       // Prefer to call requestFocusInWindow in a timer with a small delay   
       // than calling it with EnventQueue#invokeLater to ensure it always works
@@ -496,7 +514,7 @@ public class SwingTools {
       dialog.addComponentListener(new ComponentAdapter() {
           @Override
           public void componentShown(ComponentEvent ev) {
-            requestFocus(focusedComponent);
+            requestFocusInWindow(focusedComponent);
             dialog.removeComponentListener(this);
           }
         });
@@ -568,7 +586,7 @@ public class SwingTools {
   
   /**
    * Displays the image referenced by <code>imageUrl</code> in an AWT window 
-   * disposed once an other AWT frame is created.
+   * disposed once an instance of <code>JFrame</code> or <code>JDialog</code> is displayed.
    * If the <code>imageUrl</code> is incorrect, nothing happens.
    */
   public static void showSplashScreenWindow(URL imageUrl) {
@@ -590,17 +608,30 @@ public class SwingTools {
             try {
               Thread.sleep(500);
               while (splashScreenWindow.isVisible()) {
-                // If an other frame is showing, dispose splash window
                 EventQueue.invokeLater(new Runnable() {
                     public void run() {
-                      for (Frame frame : Frame.getFrames()) {
-                        if (frame.isShowing()) {
-                          splashScreenWindow.dispose();
+                      // If a JFrame or JDialog is showing, dispose splash window
+                      try {
+                        for (Window window : (Window[])Window.class.getMethod("getWindows").invoke(null)) {
+                          if ((window instanceof JFrame || window instanceof JDialog)
+                              && window.isShowing()) {
+                            splashScreenWindow.dispose();
+                            break;
+                          }
+                        }
+                      } catch (Exception ex) {
+                        // Even if splash screen will disappear quicker,
+                        // use Frame#getFrames under Java 1.5 where Window#getWindows doesn't exist
+                        for (Frame frame : Frame.getFrames()) {
+                          if (frame.isShowing()) {
+                            splashScreenWindow.dispose();
+                            break;
+                          }
                         }
                       }
                     }
                   });
-                Thread.sleep(300);
+                Thread.sleep(200);
               }
             } catch (InterruptedException ex) {
               EventQueue.invokeLater(new Runnable() {
@@ -650,6 +681,61 @@ public class SwingTools {
     return scrollPane;
   }
   
+  /**
+   * Returns a scroll bar adjustment listener bound to the given <code>scrollPane</code> view
+   * that updates view tool tip when its vertical scroll bar is adjusted.
+   */
+  public static AdjustmentListener createAdjustmentListenerUpdatingScrollPaneViewToolTip(final JScrollPane scrollPane) {
+    return new AdjustmentListener() {
+        public void adjustmentValueChanged(AdjustmentEvent ev) {
+          Point screenLocation = MouseInfo.getPointerInfo().getLocation();
+          Point point = new Point(screenLocation); 
+          Component view = scrollPane.getViewport().getView();
+          SwingUtilities.convertPointFromScreen(point, view);
+          if (scrollPane.isShowing()
+              && scrollPane.getViewport().getViewRect().contains(point)) {
+            MouseEvent mouseEvent = new MouseEvent(view, MouseEvent.MOUSE_MOVED, System.currentTimeMillis(), 
+                0, point.x, point.y, 0, false, MouseEvent.NOBUTTON);
+            if (isToolTipShowing()) {
+              ToolTipManager.sharedInstance().mouseMoved(mouseEvent);
+            }
+          }
+        }
+      };
+  }
+
+  /**
+   * Returns <code>true</code> if a tool tip is showing.
+   */
+  private static boolean isToolTipShowing() {
+    for (Frame frame : Frame.getFrames()) {
+      if (isToolTipShowing(frame)) {
+        return true;
+      }
+    }
+    return false;
+  }
+
+  private static boolean isToolTipShowing(Container container) {
+    if (container instanceof Window) {
+      for (Window window : ((Window)container).getOwnedWindows()) {
+        if (isToolTipShowing(window)) {
+          return true;
+        }
+      }
+    }
+    for (int i = 0; i < container.getComponentCount(); i++) {
+      Component child = container.getComponent(i);
+      if (child instanceof JToolTip 
+            && child.isShowing()
+          || child instanceof Container
+            && isToolTipShowing((Container)child)) {
+        return true;
+      }            
+    }
+    return false;
+  }
+
   /**
    * Adds a listener that will update the given popup menu to hide disabled menu items.
    */
@@ -811,7 +897,7 @@ public class SwingTools {
       }            
     }
   }
-
+  
   /**
    * Returns <code>true</code> if the given rectangle is fully visible at screen.
    */
@@ -862,4 +948,183 @@ public class SwingTools {
       }
     }
   }
+
+  /**
+   * Returns <code>image</code> size in pixels.
+   * @return the size or <code>null</code> if the information isn't given in the meta data of the image
+   */
+  public static Dimension getImageSizeInPixels(Content image) throws IOException {
+    InputStream in = null;
+    try {
+      in = image.openStream();
+      ImageInputStream iis = ImageIO.createImageInputStream(in);
+      Iterator<ImageReader> it = ImageIO.getImageReaders(iis);
+      if (it.hasNext()) {
+        ImageReader reader = (ImageReader)it.next();
+        reader.setInput(iis);
+        int imageWidth = reader.getWidth(reader.getMinIndex());
+        int imageHeight = reader.getHeight(reader.getMinIndex());
+        reader.dispose();
+        return new Dimension(imageWidth, imageHeight);
+      }
+      return null;
+    } finally {
+      if (in != null) {
+        in.close();
+      }
+    }
+  }
+  
+  /**
+   * Returns the line stroke matching the given line styles.
+   */
+  public static Stroke getStroke(float thickness, 
+                                 Polyline.CapStyle capStyle,
+                                 Polyline.JoinStyle joinStyle, 
+                                 Polyline.DashStyle dashStyle) {
+    int strokeCapStyle;
+    switch (capStyle) {
+      case ROUND :
+        strokeCapStyle = BasicStroke.CAP_ROUND;
+        break;
+      case SQUARE :
+        strokeCapStyle = BasicStroke.CAP_SQUARE;
+        break;
+      default:
+        strokeCapStyle = BasicStroke.CAP_BUTT;
+        break;
+    }
+    
+    int strokeJoinStyle;
+    switch (joinStyle) {
+      case ROUND :
+      case CURVED :
+        strokeJoinStyle = BasicStroke.JOIN_ROUND;
+        break;
+      case BEVEL :
+        strokeJoinStyle = BasicStroke.JOIN_BEVEL;
+        break;
+      default:
+        strokeJoinStyle = BasicStroke.JOIN_MITER;
+        break;
+    }
+    
+    float [] strokeDashes;
+    switch (dashStyle) {
+      case DOT :
+        strokeDashes = new float [] {thickness, thickness}; 
+        break;
+      case DASH :
+        strokeDashes = new float [] {thickness * 4, thickness * 2}; 
+        break;
+      case DASH_DOT :
+        strokeDashes = new float [] {thickness * 8, thickness * 2, thickness * 2, thickness * 2};
+        break;
+      case DASH_DOT_DOT :
+        strokeDashes = new float [] {thickness * 8, thickness * 2, thickness * 2, thickness * 2, thickness * 2, thickness * 2};
+        break;
+      default :
+        strokeDashes = null;
+        break;
+    }
+    
+    return new BasicStroke(thickness, strokeCapStyle, strokeJoinStyle, 10, strokeDashes, 0);
+  }
+
+  /**
+   * Updates Swing components default size according to resolution scale.
+   */
+  static void updateComponentDefaults() {
+    float resolutionScale = getResolutionScale();
+    if (resolutionScale != 1) {
+      Font buttonFont = updateComponentFontSize("Button.font", resolutionScale);
+      updateComponentFontSize("ToggleButton.font", resolutionScale);
+      updateComponentFontSize("RadioButton.font", resolutionScale);
+      updateComponentFontSize("CheckBox.font", resolutionScale);
+      updateComponentFontSize("ColorChooser.font", resolutionScale);
+      updateComponentFontSize("ComboBox.font", resolutionScale);
+      updateComponentFontSize("InternalFrame.titleFont", resolutionScale);
+      Font labelFont = updateComponentFontSize("Label.font", resolutionScale);
+      updateComponentFontSize("List.font", resolutionScale);
+      updateComponentFontSize("MenuBar.font", resolutionScale);
+      updateComponentFontSize("MenuItem.font", resolutionScale);
+      updateComponentFontSize("MenuItem.acceleratorFont", resolutionScale);
+      updateComponentFontSize("RadioButtonMenuItem.font", resolutionScale);
+      updateComponentFontSize("RadioButtonMenuItem.acceleratorFont", resolutionScale);
+      updateComponentFontSize("CheckBoxMenuItem.font", resolutionScale);
+      updateComponentFontSize("CheckBoxMenuItem.acceleratorFont", resolutionScale);
+      updateComponentFontSize("Menu.font", resolutionScale);
+      updateComponentFontSize("Menu.acceleratorFont", resolutionScale);
+      updateComponentFontSize("PopupMenu.font", resolutionScale);
+      updateComponentFontSize("OptionPane.font", resolutionScale);
+      updateComponentFontSize("Panel.font", resolutionScale);
+      updateComponentFontSize("ProgressBar.font", resolutionScale);
+      updateComponentFontSize("ScrollPane.font", resolutionScale);
+      updateComponentFontSize("Viewport.font", resolutionScale);
+      updateComponentFontSize("Slider.font", resolutionScale);
+      updateComponentFontSize("Spinner.font", resolutionScale);
+      updateComponentFontSize("Table.font", resolutionScale);
+      updateComponentFontSize("TabbedPane.font", resolutionScale);
+      updateComponentFontSize("TableHeader.font", resolutionScale);
+      updateComponentFontSize("TextField.font", resolutionScale);
+      updateComponentFontSize("FormattedTextField.font", resolutionScale);
+      updateComponentFontSize("PasswordField.font", resolutionScale);
+      updateComponentFontSize("TextArea.font", resolutionScale);
+      updateComponentFontSize("EditorPane.font", resolutionScale);
+      updateComponentFontSize("TitledBorder.font", resolutionScale);
+      updateComponentFontSize("ToolBar.font", resolutionScale);
+      updateComponentFontSize("ToolTip.font", resolutionScale);
+      updateComponentFontSize("Tree.font", resolutionScale);
+      UIManager.put("OptionPane.messageFont", labelFont);
+      UIManager.put("OptionPane.buttonFont", buttonFont);
+      int dividerSize = UIManager.getInt("SplitPane.dividerSize");
+      if (dividerSize != 0) {
+        UIManager.put("SplitPane.dividerSize", Math.round(dividerSize * resolutionScale));
+      }
+    }
+  }
+
+  private static Font updateComponentFontSize(String fontKey, float resolutionScale) {
+    Font font = UIManager.getFont(fontKey);
+    if (font != null) {
+      font = font.deriveFont(font.getSize() * resolutionScale);
+      UIManager.put(fontKey, font);
+    }
+    return font;
+  }
+  
+  /**
+   * Returns a scale factor used to adapt user interface items to screen resolution.
+   */
+  public static float getResolutionScale() {
+    String resolutionScaleProperty = System.getProperty("com.eteks.sweethome3d.resolutionScale");
+    if (resolutionScaleProperty != null) {
+      try {
+        return Float.parseFloat(resolutionScaleProperty.trim());
+      } catch (NumberFormatException ex) {
+        // Ignore resolution
+      }
+    }
+    
+    return 1f;
+  }
+  
+  /**
+   * Returns an image icon scaled according to the value returned by {@link #getResolutionScale()}.
+   */
+  public static ImageIcon getScaledImageIcon(URL imageUrl) {
+    float resolutionScale = getResolutionScale();
+    if (resolutionScale == 1) {
+      return new ImageIcon(imageUrl);
+    } else {
+      try {
+        BufferedImage image = ImageIO.read(imageUrl);
+        Image scaledImage = image.getScaledInstance(Math.round(image.getWidth() * resolutionScale), 
+            Math.round(image.getHeight() * resolutionScale), Image.SCALE_SMOOTH);
+        return new ImageIcon(scaledImage);
+      } catch (IOException ex) {
+        return null;
+      }
+    }
+  }
 }
