@@ -136,6 +136,7 @@ import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.tools.ResourceURLContent;
 import com.eteks.sweethome3d.viewcontroller.ContentManager;
 import com.eteks.sweethome3d.viewcontroller.DialogView;
+import com.eteks.sweethome3d.viewcontroller.Object3DFactory;
 import com.eteks.sweethome3d.viewcontroller.VideoController;
 import com.eteks.sweethome3d.viewcontroller.View;
 
@@ -169,6 +170,7 @@ public class VideoPanel extends JPanel implements DialogView {
 
   private final Home            home;
   private final UserPreferences preferences;
+  private final Object3DFactory object3dFactory;
   private final VideoController controller;
   private PlanComponent         planComponent; 
   private JToolBar              videoToolBar;
@@ -203,14 +205,22 @@ public class VideoPanel extends JPanel implements DialogView {
   private static VideoPanel     currentVideoPanel; // Support only one video panel opened at a time
 
   /**
-   * Creates a video panel.
+   * Creates a video panel with default object 3D factory.
    */
   public VideoPanel(Home home, 
                     UserPreferences preferences, 
                     VideoController controller) {
+    this(home, preferences, null, controller);
+  }
+  
+  public VideoPanel(Home home, 
+                    UserPreferences preferences, 
+                    Object3DFactory object3dFactory,
+                    VideoController controller) {
     super(new GridBagLayout());
     this.home = home;
     this.preferences = preferences;
+    this.object3dFactory = object3dFactory;
     this.controller = controller;
     createActions(preferences);
     createComponents(home, preferences, controller);
@@ -719,8 +729,8 @@ public class VideoPanel extends JPanel implements DialogView {
     timeSpinnerModel.addChangeListener(dateTimeChangeListener);
 
     this.dayNightLabel = new JLabel();
-    final ImageIcon dayIcon = new ImageIcon(PhotoPanel.class.getResource("resources/day.png"));
-    final ImageIcon nightIcon = new ImageIcon(PhotoPanel.class.getResource("resources/night.png"));
+    final ImageIcon dayIcon = SwingTools.getScaledImageIcon(PhotoPanel.class.getResource("resources/day.png"));
+    final ImageIcon nightIcon = SwingTools.getScaledImageIcon(PhotoPanel.class.getResource("resources/night.png"));
     PropertyChangeListener dayNightListener = new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent ev) {
           if (home.getCompass().getSunElevation(
@@ -990,8 +1000,8 @@ public class VideoPanel extends JPanel implements DialogView {
       Point dialogLocation = null;
       if (homeRoot != null) {
         // Restore location if it exists
-        Integer x = (Integer)this.home.getVisualProperty(VIDEO_DIALOG_X_VISUAL_PROPERTY);
-        Integer y = (Integer)this.home.getVisualProperty(VIDEO_DIALOG_Y_VISUAL_PROPERTY);      
+        Number x = this.home.getNumericProperty(VIDEO_DIALOG_X_VISUAL_PROPERTY);
+        Number y = this.home.getNumericProperty(VIDEO_DIALOG_Y_VISUAL_PROPERTY);      
 
         int windowRightBorder = homeRoot.getX() + homeRoot.getWidth();
         Dimension screenSize = getToolkit().getScreenSize();
@@ -1010,9 +1020,9 @@ public class VideoPanel extends JPanel implements DialogView {
         }
         int dialogHeight = dialog.getHeight();
         if (x != null && y != null 
-            && x + dialogWidth <= screenRightBorder
-            && y + dialogHeight <= screenBottomBorder) {
-          dialogLocation = new Point(x, y);
+            && x.intValue() + dialogWidth <= screenRightBorder
+            && y.intValue() + dialogHeight <= screenBottomBorder) {
+          dialogLocation = new Point(x.intValue(), y.intValue());
         } else if (screenRightBorder - windowRightBorder > dialogWidth / 2
             || dialogHeight == screenHeight) {
           // If there some space left at the right of the window
@@ -1057,8 +1067,8 @@ public class VideoPanel extends JPanel implements DialogView {
 
           @Override
           public void componentMoved(ComponentEvent ev) {
-            controller.setVisualProperty(VIDEO_DIALOG_X_VISUAL_PROPERTY, dialog.getX());
-            controller.setVisualProperty(VIDEO_DIALOG_Y_VISUAL_PROPERTY, dialog.getY());
+            controller.setHomeProperty(VIDEO_DIALOG_X_VISUAL_PROPERTY, String.valueOf(dialog.getX()));
+            controller.setHomeProperty(VIDEO_DIALOG_Y_VISUAL_PROPERTY, String.valueOf(dialog.getY()));
           }
         });
       dialog.setVisible(true);
@@ -1252,8 +1262,9 @@ public class VideoPanel extends JPanel implements DialogView {
    */
   private Camera [] getVideoFramesPath(int frameRate) {
     List<Camera> videoFramesPath = new ArrayList<Camera>();
-    final float moveDistancePerFrame = 240000f / 3600 / frameRate;  // 3 cm/frame = 1800 m / 3600 s / 25 frame/s = 2.4 km/h
-    final float moveAnglePerFrame = (float)(Math.PI / 180 * 30 / frameRate);
+    final float speed = 2400f / 3600; // 2.4 km/h
+    final float moveDistancePerFrame = speed * 100f / frameRate;  // speed is in m/s
+    final float moveAnglePerFrame = (float)(Math.PI / 120 * 30 * speed / frameRate);
     final float elapsedTimePerFrame = 345600 / frameRate * 25; // 250 frame/day at 25 frame/second
     
     List<Camera> cameraPath = this.controller.getCameraPath();
@@ -1378,11 +1389,12 @@ public class VideoPanel extends JPanel implements DialogView {
     try {
       file = OperatingSystem.createTemporaryFile("video", ".mov"); 
       if (quality >= 2) {
-        frameGenerator = new PhotoImageGenerator(home, width, height, quality == 2 
+        frameGenerator = new PhotoImageGenerator(home, width, height, this.object3dFactory, 
+            quality == 2 
               ? PhotoRenderer.Quality.LOW
               : PhotoRenderer.Quality.HIGH);        
       } else {
-        frameGenerator = new Image3DGenerator(home, width, height, quality == 1); 
+        frameGenerator = new Image3DGenerator(home, width, height, this.object3dFactory, quality == 1); 
       }
       if (!Thread.currentThread().isInterrupted()) {
         ImageDataSource sourceStream = new ImageDataSource((VideoFormat)this.videoFormatComboBox.getSelectedItem(), 
@@ -1780,9 +1792,10 @@ public class VideoPanel extends JPanel implements DialogView {
     private PhotoRenderer renderer;
     private BufferedImage image;
     
-    public PhotoImageGenerator(Home home, int width, int height, 
+    public PhotoImageGenerator(Home home, int width, int height,
+                               Object3DFactory object3dFactory,
                                PhotoRenderer.Quality quality) throws IOException {
-      this.renderer = new PhotoRenderer(home, quality); 
+      this.renderer = new PhotoRenderer(home, object3dFactory, quality); 
       this.image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
     }
 
@@ -1812,10 +1825,11 @@ public class VideoPanel extends JPanel implements DialogView {
     private HomeComponent3D homeComponent3D;
     private BufferedImage   image;
 
-    public Image3DGenerator(Home home, int width, int height, 
+    public Image3DGenerator(Home home, int width, int height,
+                            Object3DFactory object3dFactory, 
                             boolean displayShadowOnFloor) {
       this.home = home;
-      this.homeComponent3D = new HomeComponent3D(home, null, displayShadowOnFloor);
+      this.homeComponent3D = new HomeComponent3D(home, null, object3dFactory, displayShadowOnFloor, null);
       this.homeComponent3D.startOffscreenImagesCreation();
       this.image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
     }
